/*
** Module   : terminal.c
** Abstract : Generic implementation for character mode-based console functions.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 EVL 20130723 Created initial version to implement text console terminal JNI.
*/

#include "terminal.h"

#include "com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper.h"

int base_rows = 0;
int base_cols = 0;

/* This function outputs the screenful of data at once using an array of codes
   where every character position is either a skip indicator (-1) or a fully 
   prepared code with character, attribute and color included.
   The final step is the cursor positioning.
*/
JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_addArrayNative
  (JNIEnv *env, jobject jo, jintArray arr, jint cx, jint cy)
{
   addArrayNative(env, jo, arr, cx, cy);
}

/*
 * Draw a string of text.
 */
JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_addStringNative
  (JNIEnv *env, jobject jo, jbyteArray jstr, jint len, jint attrib, jint colorpair)
{
   addStringNative(env, jo, jstr, len, attrib, colorpair);
}

/*
 * Ring the terminal bell.
 */
JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_beep
  (JNIEnv *env, jobject jo)
{
   beepNative(env, jo);
}

JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_clear
  (JNIEnv *env, jobject jo)
{
   clearNative(env, jo);
}

JNIEXPORT jint JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_getACSchar
  (JNIEnv *env, jclass class, jint offset)
{
   return getACSchar(env, class, offset);
}

JNIEXPORT jint JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_getAttribute
  (JNIEnv *env, jclass class, jint offset)
{
   return getAttribute(env, class, offset);
}

/* Return the number of columns that the screen can display.
* The ncurses does not return a reliable value for xterm windows,
* where the screen size can change at runtime.
 */
JNIEXPORT jint JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_getScreenColumnsNative
  (JNIEnv *env, jobject jo)
{
   return (jint) base_cols;
}

/* Return the number of rows that the screen can display.
 */
JNIEXPORT jint JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_getScreenRowsNative
  (JNIEnv *env, jobject jo)
{
   return (jint) base_rows;
}

// return the NCURSES terminal type
JNIEXPORT jstring JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_getTermType
  (JNIEnv *env, jclass  cls)
{
   return getTermType(env, cls);
}

/* Get the x position of the cursor.
 */
JNIEXPORT jint JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_getx
  (JNIEnv *env, jobject jo)
{
   return getx(env, jo);
}

/* Get the y position of the cursor.
 */
JNIEXPORT jint JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_gety
  (JNIEnv *env, jobject jo)
{
   return gety(env, jo);
}

/* Returns true if the terminal is capable of displaying colors.
 */
JNIEXPORT jboolean JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_hasColors
  (JNIEnv *env, jobject jo)
{
   return hasColors(env, jo);
}

JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_initConsole(JNIEnv *env, jobject jo)
{
   initConsole(env, jo);
}

// Returns -1 if there was no character to read.
// Otherwise returns the character that was read.
JNIEXPORT jint JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_readKey
  (JNIEnv *env, jobject jo)
{
   return readKey(env, jo);
}

JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_resetMode
  (JNIEnv *env, jobject jo)
{
   resetMode(env, jo);
}

JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_resume
  (JNIEnv *env, jobject jo)
{
   resume(env, jo);
}

JNIEXPORT jboolean JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_setCursorNative
  (JNIEnv *env, jobject jo, jint x, jint y)
{
   return setCursorNative(env, jo, x, y);
}

JNIEXPORT jboolean JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_setCursorStatus
  (JNIEnv *env, jobject jo, jboolean on)
{
   return setCursorStatus(env, jo, on);
}

// set the NCURSES terminal type
JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_setTermType(JNIEnv *env,
                                                           jclass  cls,
                                                           jstring name)
{
   setTermType(env, cls, name);
}

JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_suspend
  (JNIEnv *env, jobject jo)
{
   suspend(env, jo);
}

JNIEXPORT void JNICALL Java_com_goldencode_p2j_ui_client_chui_driver_console_ConsoleHelper_syncNative
   (JNIEnv *env, jobject jo)
{
   syncNative(env, jo);
}
